# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import gtk
import brisa
import os

class BrisaTray(object):

    def __init__(self):
        self.create_main_icon()
        self.base_dir = os.path.dirname(__file__)
        self.base_dir = os.path.join(self.base_dir, 'images')

    def make_main_menu(self, event_button, event_time, icon):

        menu = gtk.Menu()

        ms_item = gtk.MenuItem('Media Server')
        ms_item.set_submenu(self.create_ms_menu())

        mr_item = gtk.MenuItem('Media Renderer')
        mr_item.set_submenu(self.create_mr_menu())

        cp_item = gtk.MenuItem('Control Point')
        cp_item.connect('activate', self.launch_cp)

        separator = gtk.MenuItem(None)

        about_item = gtk.ImageMenuItem(gtk.STOCK_ABOUT)
        about_item.connect('activate', self.about)

        quit_item = gtk.ImageMenuItem(gtk.STOCK_QUIT)
        quit_item.connect('activate', gtk.main_quit)

        menu.append(ms_item)
        menu.append(mr_item)
        menu.append(cp_item)
        menu.append(about_item)
        menu.append(separator)
        menu.append(quit_item)

        for m_item in menu:
            m_item.show()

        menu.popup(None, None, gtk.status_icon_position_menu,
                   event_button, event_time, icon)

    def create_ms_menu(self):
        app = '/usr/bin/brisa-server'
        ms_menu_list = [('start', gtk.MenuItem('Start'), self.ms_start),
                        ('stop', gtk.MenuItem('Stop'), self.ms_stop),
                        ('restart', gtk.MenuItem('Restart'), self.ms_restart),
                        ('sep', gtk.MenuItem(None), None),
                        ('configure', gtk.ImageMenuItem(gtk.STOCK_PROPERTIES),
                         self.ms_configure)]
        return self.create_submenu(app, ms_menu_list)

    def create_submenu(self, app, m_list):
        submenu = gtk.Menu()
        for item in m_list:
            if item[0] == 'start':
                if self.get_pid(app):
                    continue
            elif item[0] == 'stop':
                if not self.get_pid(app):
                    continue
            submenu.append(item[1])
            item[1].show()
            if item[2]:
                item[1].connect('activate', item[2])
        return submenu

    def create_mr_menu(self):
        app = '/usr/bin/brisa-renderer'
        mr_menu_list = [('start', gtk.MenuItem('Start'), self.mr_start),
                        ('stop', gtk.MenuItem('Stop'), self.mr_stop),
                        ('restart', gtk.MenuItem('Restart'), self.mr_restart)]
        return  self.create_submenu(app, mr_menu_list)

    def ms_start(self, widget):
        app = '/usr/bin/brisa-server'
        if not self.get_pid(app):
            self.launch_app(app, ['', '--daemon'])

    def ms_stop(self, widget):
        app = '/usr/bin/brisa-server'
        self.kill_process(self.get_pid(app))

    def ms_restart(self, widget):
        self.ms_stop(None)
        self.ms_start(None)

    def mr_start(self, widget):
        app = '/usr/bin/brisa-renderer'
        if not self.get_pid(app):
            self.launch_app(app, ['', '--daemon'])

    def ms_configure(self, widget):
        self.launch_app('/usr/bin/brisa-localconf')

    def mr_stop(self, widget):
        app = '/usr/bin/brisa-renderer'
        self.kill_process(self.get_pid(app))

    def mr_restart(self, widget):
        self.mr_stop(None)
        self.mr_start(None)

    def about(self, widget):
        window = gtk.AboutDialog()
        window.set_name("BRisa UPnP A/V Framework")
        window.set_version(brisa.__version__)
        window.set_copyright("\302\251 Copyright 2007-2008 BRisa Team")
        window.set_website("http://brisa.garage.maemo.org/")
        window.connect("response", lambda d, r: d.destroy())
        window.show()

    def launch_app(self, app, args=['']):
        try:
            pid = os.fork()
            if pid == 0:
                os.execv(app, args)
            else:
                os.wait()
        except OSError, e:
            pass

    def launch_cp(self, widget):
        self.launch_app('/usr/bin/brisa-controlpoint')

    def show(self):
        gtk.main()

    def on_right_click(self, icon, event_button, event_time):
        self.make_main_menu(event_button, event_time, icon)

    def create_main_icon(self):
        icon= gtk.status_icon_new_from_icon_name('brisa2')
        icon.connect('popup-menu', self.on_right_click)

    def get_pid(self, prog):
        pid_list = []
        for p in os.listdir("/proc"):
            path = "/proc/%s/cmdline" % p
            try:
                cmdline = open(path).read()
            except (IOError, OSError), e:
                continue
            if prog in cmdline:
                pid_list.append(int(p))
        return pid_list

    def kill_process(self, pid_list):
        for p in pid_list:
            os.kill(p, 15)

if __name__ == '__main__':
    tray = BrisaTray()
    tray.show()

