# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

from xml.etree import ElementTree

from brisa import log
from brisa.main import ThreadManager
from brisa.services.web_server import WebServer
from brisa.services.web_server import CustomResource
from brisa.utils.network import get_active_ifaces
from brisa.utils.network import get_ip_address


class EventListener(CustomResource):
    isLeaf = True

    def __init__(self, observer):
        CustomResource.__init__(self)
        self.observer = observer

    def render(self, uri, request, response):
        log.debug('Received render (%s)' % str((uri, request, response)))

        if request.method == 'NOTIFY':
            log.debug('Ok, got notify!')
            self.render_NOTIFY(request, response)
        else:
            log.debug('Did not get notify, got %s' % request.method)

        log.debug('Returning from render')

    def render_NOTIFY(self, request, response):
        data = request.body
        ThreadManager().run_async_function(self.forward_notification,
                                           (request.headers, data),
                                           0.0001)
        return ""

    def forward_notification(self, received_headers, data):
        headers = {}
        changed_vars = {}
        for k, v in received_headers.items():
            headers[k.lower()] = v.lower()
        try:
            tree = ElementTree.XML(data)
        except:
            tree = None

        if tree:
            for prop1 in tree.findall('.//{%s}property' %
                                      'urn:schemas-upnp-org:event-1-0'):
                # prop1 = <e:property> <Ble> cont </Ble> </e:property>
                for prop2 in prop1:
                    # <Ble>cont</Ble>
                    changed_vars[prop2.tag] = prop2.text

        if self.observer and 'sid' in headers:
            self.observer._on_event(headers['sid'], changed_vars)


class EventListenerServer(object):

    def __init__(self, observer):
        self.srv = WebServer()
        self.srv.add_resource('', EventListener(observer))
        self.srv.start()

    def host(self):
        host = self.srv.get_host()
        port = self.srv.get_port()

        log.debug('EventListenerServer returning %s' % str((host, port)))
        return (host, port)

    def stop(self):
        if self.srv:
            self.srv.stop()
