# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import sys

from brisa.control_point.control_point import ControlPoint
from brisa.main import ThreadManager, BRISA_PROCESSES


class Observer(object):
    # Needed callbacks
    l = []

    def on_new_device(self, dev):
        self.l.append(dev)

    def on_device_removed(self, udn):
        for dev in self.l:
            if dev.udn == udn:
                self.l.remove(dev)
                break

    def list_devices(self):
        for dev in self.l:
            print 'device:'
            print '\tudn:', dev.udn
            print '\tfriendly_name:', dev.friendly_name
            print '\tservices:', dev.services
            print '\ttype:', dev.device_type
            if dev.devices:
                print '\tchild devices:'
                for child_dev in dev.devices:
                    print '\t\tudn:', child_dev.udn
                    print '\t\tfriendly_name:', child_dev.friendly_name
                    print '\t\tservices:', dev.services
                    print '\t\ttype:', child_dev.device_type
            print


def main():
    exit = False

    obs = Observer()
    cp = ControlPoint()
    cp.subscribe("new_device_event", obs.on_new_device)
    cp.subscribe("removed_device_event", obs.on_device_removed)

    try:
        while not exit:
            c = str(raw_input('>>> '))

            if c == 'exit':
                exit = True
            elif c == 'help':
                print 'commands: exit, help, start_search, stop_search, list'
            elif c == 'start_search':
                cp.start_search(600, "upnp:rootdevice")
                print 'search started!'
            elif c == 'stop_search':
                cp.stop_search()
                print 'search stopped!'
            elif c == 'list':
                print obs.list_devices()

            c = ''
    except KeyboardInterrupt, k:
        print 'quiting'
        ThreadManager().stop_all(BRISA_PROCESSES)
        sys.exit(0)

    ThreadManager().stop_all(BRISA_PROCESSES)
    sys.exit(0)

if __name__ == "__main__":
    main()
