#!/usr/bin/env python
# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

from os import path
from threading import Timer
from time import sleep

import gtk
import gtk.glade

from launch_api import *
from brisa.config import conf
from brisa.main import ThreadObject, ThreadManager


# Find a way when packaging to export these paths! perhaps put them in conf
media_server_app = '/usr/bin/brisa-media-server'
media_server_name = 'brisa-media-server'

media_renderer_app = '/usr/bin/brisa-media-renderer'
media_renderer_name = 'brisa-media-renderer'
    
    
class LaunchTool(object):
    location = path.dirname(__file__)
    
    def __init__(self):
        gladefile = path.join(self.location, 'launch_tool.glade')
        self.wTree = gtk.glade.XML(gladefile)
        self.window = self.wTree.get_widget('launcherMainWindow')
        self.window.connect('destroy', self.quit)
        self.window.show()
        
        events = {'on_media_server_start': self.start_media_server,
                   'on_media_server_stop': self.stop_media_server,
                   'on_media_renderer_start': self.start_media_renderer,
                   'on_media_renderer_stop': self.stop_media_renderer}
        
        self.wTree.signal_autoconnect(events)
        self.refresh_media_server_status()
        self.refresh_media_renderer_status()

    def start_media_server(self, widget):
        start_server(media_server_name, media_server_app)
        self.schedule_status_refresh(self.refresh_media_server_status, 0.5)
    
    def stop_media_server(self, widget):
        stop_server(media_server_name)
        self.schedule_status_refresh(self.refresh_media_server_status, 0.5)
        
    def start_media_renderer(self, widget):
        start_server(media_renderer_name, media_renderer_app)
        self.schedule_status_refresh(self.refresh_media_renderer_status, 0.5)
    
    def stop_media_renderer(self, widget):
        stop_server(media_renderer_name)
        self.schedule_status_refresh(self.refresh_media_renderer_status, 0.5)
        
    def refresh_media_server_status(self):
        status = get_status(media_server_name)
        label = self.get_media_server_label()
        try:
            if label.get_text() != status:
                label.set_text('Status: %s' % status)
        except:
            pass
    
    def refresh_media_renderer_status(self):
        status = get_status(media_renderer_name)
        label = self.get_media_renderer_label()
        try:
            if label.get_text() != status:
                label.set_text('Status %s' % status)
        except:
            pass
    
    def refresh_all(self):
        self.refresh_media_server_status()
        self.refresh_media_renderer_status()
        
    def get_media_server_label(self):
        return self.wTree.get_widget("mediaServerLabel")
    
    def get_media_renderer_label(self):
        return self.wTree.get_widget("mediaRendererLabel")
    
    def schedule_status_refresh(self, f, d):
        t = Timer(d, f, ())
        t.start()
        
    def show(self):
        gtk.gdk.threads_init()
        gtk.main()
        
    def quit(self, widget):
        gtk.gdk.threads_leave()
        gtk.main_quit()
        ThreadManager().stop_all()

class Refresh(ThreadObject):

    def __init__(self, launch_tool_instance):
        ThreadObject.__init__(self)
        self.launch_tool = launch_tool_instance
        
    def run(self):
        while self.is_running():
            self.launch_tool.refresh_all()
            sleep(4)        
        
def main():
    launch_tool = LaunchTool()
    r = Refresh(launch_tool)
    r.start()
    launch_tool.show()
