# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

from cherrypy.lib.static import serve_file

from brisa.services.web_server.resource import CustomResource

class StaticFile(object):
    # Attribute for cherrypy's publishing
    exposed = True

    def __init__(self, path):
        self.path = path

    def render(self, uri=None, request=None, response=None):
        """ Method that renders a request and returns the response.
        """
        return serve_file(self.path)

    def __call__(self, *args, **kwargs):
        return self.render(*args, **kwargs)

    def __str__(self):
        return "<StaticFile, Path: %s>" % self.path


class Data(CustomResource):

    def __init__(self, data, type):
        CustomResource.__init__(self)
        self.data = data
        self.type = type

    def render(self, uri, request, response):
        request.headers["content-type"] = self.type
        request.headers["content-length"] = str(len(self.data))
        if request.method == "HEAD":
            return ''
        return self.data

    def __str__(self):
        return '<Data data: %s, type: %s>' % (str(self.data), str(self.type))
