# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.

# Copyright 2006, Elisa team <contact@fluendo.com>
# Copyright 2007-2008, Brisa team <brisa-develop@garage.maemo.org>

import uuid

class Device(object):

    def __init__(self, device_type, friendly_name, location, udn=None,
                 parent=None):
        self.parent = parent
        self.location = location
        self.friendly_name = friendly_name
        self.manufacturer = ""
        self.manufacturer_url = ""
        self.model_description = ""
        self.model_name = ""
        self.model_number = ""
        self.model_url = ""
        self.serial_number = ""
        self.upc = ""
        self.presentation_url = ""
        self.device_type = device_type

        self.services = {}
        self.icons = []
        self.webResourceList = []

        if udn == None:
            self.udn = self.generate_udn()
        else:
            self.udn = udn

    def generate_udn(self):
        usn = 'uuid:%s' % uuid.uuid4()
        return usn

    def remove_service_with_usn(self, service_usn):
        for k, service in self.services.items():
            if service.get_usn() == service_usn:
                del self.services[k]
                break

    def is_root_device(self):
        return isinstance(self, RootDevice)

class DeviceIcon(object):

    def __init__(self, mimetype, width, height, depth, url):
        self.mimetype = mimetype
        self.width = width
        self.height = height
        self.depth = depth
        self.url = url

    def get_mimetype(self):
        return self.mimetype

    def get_width(self):
        return  self.width

    def get_height(self):
        return self.height

    def get_depth(self):
        return self.depth

    def get_url(self):
        return self.url

class RootDevice(Device):

    def __init__(self, device_type, friendly_name, location, udn=None,
                 parent=None):
        Device.__init__(self, device_type, friendly_name, location, udn,
                        parent)
        self.devices = []

class ChildDevice(Device):
    def __init__(self, type, friendly_name, location, udn=None,
                 parent=None):
        Device.__init__(self, type, friendly_name, location, udn,
                        parent)
        self.devices = []
