# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

# Implementation of a simple API for use in our UPnP Framework
import os
from os import path

from brisa.upnp.message_handle.device_description_message import\
DeviceDescriptionMessage
from brisa.upnp.ssdp import SSDPServer
from brisa.main import ThreadManager
from brisa import config
from brisa import log

class DeviceHandler(object):

    def __init__(self, device_name):
        self.device_name = device_name
        self.SSDP = None
        self.device = None

    def config_device(self, device):
        """Configure Webserver.

        Configure the Webserver and the SSDP server of the UPnP Device"""
        self.device = device
        ddesc = DeviceDescriptionMessage(device, device.location)
        xml_file_name = "%s-root-device.xml" % self.device_name
        xml_file_name = xml_file_name.replace(" ", "")
        xml_file_path = path.join(config.conf.brisa_home, 'tmp_xml')
        if not path.exists(xml_file_path):
            os.mkdir(xml_file_path)
        xml_file_path = path.join(xml_file_path, xml_file_name)
        ddesc.generate_xml(xml_file_path)
        self.SSDP = SSDPServer(self.device_name, xml_file_name)
        self.SSDP.register_root_device(device)
        return (xml_file_name, xml_file_path)

    def start_device(self):
        self.SSDP.announce_device(self.device)

    def stop_device(self):
        log.debug("DeviceHandler: stopping device")
        if self.SSDP:
            self.SSDP.doStop()
