# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>

import socket
from threading import Timer

from brisa import log
from brisa.main import ThreadManager


class UDPTransport:

    def __init__(self, TTL=2):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM,
                                    socket.IPPROTO_UDP)
        self.set_TTL(TTL)

    def set_TTL(self, TTL):
        self.socket.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, TTL)

    def send_data(self, data, (host, port)):
        ThreadManager().run_async_function(self.socket.sendto,
                                           (data, (host, port)))

    def send_delayed(self, delay, data, (host, port)):
        t = Timer(delay, self.send_data, args=[data, (host, port)])
        t.start()


class TCPTransport:

    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

    def send_data(self, data, (host, port)):
        ThreadManager().run_async_function(self._send_data,
                                           (data, (host, port)))

    def _send_data(self, data, (host, port)):
        try:
            self.socket.connect((host, port))
            self.socket.send(data)
            self.socket.close()
        except socket.error, e:
            log.debug('TCPTransport: couldn\'t connect to %s:%d to send data \
                    - %s' % (host, port, e.message))
            return

    def connect_and_feed(self, feeder, (host, port)):
        """ Feed -> generator """
        try:
            self.socket.connect((host, port))
            for feed in feeder.next():
                self.socket.send(feed)
            self.socket.close()
        except socket.error, e:
            log.debug('TCPTransport: error while feeding socket - %s'
                      % e.message)
