# Licensed under the MIT license
# http://opensource.org/licenses/mit-license.php or see LICENSE file.
# Copyright 2007-2008 Brisa Team <brisa-develop@garage.maemo.org>


def gen_property_with_default(name, fget=None, fset=None, doc=""):
    if fget == None and fset == None:
        raise NotImplementedError("fget or fset must be not null")

    internal_name = '%s%s' % ("_prop_", name)

    def getter(self):
        if not internal_name in dir(self):
            setattr(self, internal_name, "")
        return getattr(self, internal_name)

    def setter(self, value):
        return setattr(self, internal_name, value)

    if fget is None:
        return property(getter, fset, doc=doc)

    return property(fget, setter, doc=doc)


def gen_property_of_type(name, _type, doc=""):
    internal_name = '%s%s' % ("_prop_", name)

    def getter(self):
        return getattr(self, internal_name)

    def setter(self, value):
        if isinstance(value, _type):
            return setattr(self, internal_name, value)
        else:
            raise TypeError(("invalid type '%s' for property %s:"
                             "%s is required.") %
                            (type(value).__name__, name, type(_type).__name__))

    return property(getter, setter, doc=doc)
